#include <iostream>
#include "bitmask.h"
#include "geometry.h"


namespace std
{
  ostream&
  operator<<(ostream &ostr, const sudoku::Bitmask &bitmask)
  {
    ostr << '[';

    sudoku::bitmask_t bits = bitmask.getBits();
    if (bits != 0)
    {
      unsigned i = 0;

      do
      {
        if ((bits & 1) != 0)
        {
          ostr << i;

          if ((bits & static_cast<sudoku::bitmask_t>(~1)) != 0)
            ostr << ',';
        }

        ++i;
        bits >>= 1;
      }
      while (bits != 0);
    }

    ostr << ']';

    return ostr;
  }
}


namespace sudoku
{

unsigned
Bitmask::countBits(bitmask_t bits)
{
  unsigned count = 0;

  while (bits != 0)
  {
    if ((bits & 1) != 0)
      ++count;
    bits >>= 1;
  } /* while */

  return count;
}


Bitmask::Bitmask(size_type maxNumber)
  : m_bits(Geometry::getNumberMask(maxNumber))
  , m_count(maxNumber)
{}


Bitmask::Bitmask(const bitmask_t bits)
  : m_bits(bits)
  , m_count(countBits(bits))
{}


bool
Bitmask::setBit(unsigned i)
{
  const bitmask_t mask = (1 << i);
  const bool isChanging = ((getBits() & mask) == 0);

  if (isChanging)
  {
    m_bits |= mask;
    ++m_count;
  }

  return isChanging;
}


bool
Bitmask::clearBit(unsigned i)
{
  const bitmask_t mask = (1 << i);
  const bool isChanging = ((getBits() & mask) != 0);

  if (isChanging)
  {
    m_bits &= ~mask;
    --m_count;
  }

  return isChanging;
}


unsigned
Bitmask::getFirstBit() const
{
  unsigned i = 0;
  bitmask_t bits = getBits();

  while ((bits >>= 1) != 0)
    ++i;

  return i;
}


Bitmask&
Bitmask::operator=(const Bitmask &b)
{
  if (this != &b)
  {
    m_bits = b.m_bits;
    m_count = b.m_count;
  }

  return *this;
}

}

