#ifndef BITMASK_H
#define BITMASK_H

#include "constants.h"

namespace sudoku
{
  class Bitmask
  {
  public:
    typedef unsigned size_type;

  private:
    bitmask_t  m_bits;
    size_type  m_count;

  public:
    explicit Bitmask(size_type maxNumber=0);

    explicit Bitmask(const bitmask_t bits);

    void clear() { m_bits = 0; m_count = 0; }

    bitmask_t getBits() const { return m_bits; }
   // bitmask_t& getBits() { return m_bits; }

    size_type getCount() const { return m_count; }

    bool setBit(unsigned i);
    bool clearBit(unsigned i);
    bool isSet(unsigned i) const { return ((getBits() & (1 << i)) != 0); }
    bool isClear(unsigned i) const { return !isSet(i); }

    unsigned getFirstBit() const;

    static unsigned countBits(bitmask_t);

    bool operator==(const Bitmask &b) const {
      return (m_bits == b.m_bits);
    }

    Bitmask& operator=(const Bitmask&);
  };

  class Subset
  {
  private:
    Bitmask m_squares;
    Bitmask m_numbers;

  public:
    void setSquares(const Bitmask &b) { m_squares = b; }
    const Bitmask& getSquares() const { return m_squares; }

    const Bitmask& getNumbers() const { return m_numbers; }
    Bitmask& getNumbers() { return m_numbers; }

    void addNumber(unsigned n) { m_numbers.setBit(n); }

    void clear() { m_squares.clear(); m_numbers.clear(); }

    bool isClosed() const {
      return (m_squares.getCount() == m_numbers.getCount());
    }
  };
}

namespace std
{
  ostream& operator<<(ostream&, const sudoku::Bitmask&);
}

#endif

