#include "freedom.h"
#include "sequence.h"

namespace sudoku
{

Freedom::Freedom(Sequence *s)
  : m_size(s->size() + 1)	
  , m_sequence(s)
{
  m_bitmask = new Bitmask[m_size];
}


Freedom::~Freedom()
{
  delete[] m_bitmask;
}


bool
Freedom::findNumberSubsets(SubsetCollection &subsets) const
{
  Subset subset;

  for (size_type n = size() - 1; n > 1; --n)
  {
    subset.setSquares( m_bitmask[n] );
    subset.getNumbers().clear();
    subset.addNumber(n);

    for (size_type m = n - 1; m != 0; --m)
    {
      if (m_bitmask[n] == m_bitmask[m])
      {
        subset.addNumber(m);
      }
    } /* for */

    if (subset.isClosed())
    {
      subsets.push_back(subset);
      break;
    }
  } /* for */

  return !subsets.empty();
}

}


namespace std
{

ostream&
operator<<(ostream &ostr, const sudoku::Freedom &f)
{
  for (sudoku::Freedom::size_type i = 1; i < f.size(); ++i)
  {
    ostr << i << ':' << f[i] << endl;
  } /* for */

  return ostr;
}

}

