#include <sstream>
using namespace std;

#include "geometry.h"
#include "strip_blanks.h"

namespace sudoku
{

/*********************************************************************
 */
Geometry::Geometry(size_type gridWidth, size_type gridHeight)
  : m_maxNumber( getMaxNumber(gridWidth, gridHeight) )
  , m_gridWidth(gridWidth)
  , m_gridHeight(gridHeight)
{
  verify(m_maxNumber);
}


/*********************************************************************
 */
void
Geometry::setGridWidth(size_type w)
{
  m_maxNumber = verify(m_gridHeight * w);
  m_gridWidth = w;
}

/*********************************************************************
 */
void
Geometry::setGridHeight(size_type h)
{
  m_maxNumber = verify(m_gridWidth * h);
  m_gridHeight = h;
}

/*********************************************************************
 */
Geometry::size_type
Geometry::verify(size_type maximum)
{
  if (maximum > MAX_GRID_SIZE)
  {
    ostringstream ostr;
    ostr << "Grid too large: maximum of "
         << MAX_GRID_SIZE
         << " distinct numbers.";
    throw runtime_error(ostr.str());
  }

  return maximum;
}

/*********************************************************************
 */
bool
Geometry::operator==(const Geometry &g) const
{
  return ( (g.getGridWidth() == getGridWidth()) &&
           (g.getGridHeight() == getGridHeight()) );
}

}

namespace std
{

/*********************************************************************
 */
istream&
operator>>(istream &istr, sudoku::Geometry &geometry)
{
  const streampos pos = istr.tellg();
  bool failed = true;
  string line;

  if ( !sudoku::getLine(istr, line).empty() )
  {
    sudoku::StringPair option;

    if ( sudoku::getOption(line, option) &&
         (option.name.compare("GEOMETRY") == 0) )
    {
      istringstream is(option.value);

      sudoku::Geometry::size_type width, height;
      char c;

      is >> width >> ws >> c >> ws >> height;
      if (is && (width != 0) && (height != 0))
      {
        geometry.setGridWidth(width);
        geometry.setGridHeight(height);
        failed = false;
      }
    }
  }

  if (failed)
    istr.seekg(pos, ios::beg);

  return istr;
}

/*********************************************************************
 */
ostream&
operator<<(ostream &ostr, const sudoku::Geometry &geometry)
{
  ostr << geometry.getGridWidth()
       << "x"
       << geometry.getGridHeight();
  return ostr;
}

}

