#ifndef GEOMETRY_H
#define GEOMETRY_H

#include <climits>
#include <iostream>
#include "constants.h"

namespace sudoku
{
  class Geometry
  {
  public:
    typedef unsigned  size_type;

    static const size_type MAX_GRID_SIZE = (sizeof(size_type)*CHAR_BIT - 1);

    static const size_type DEFAULT_WIDTH = 3;
    static const size_type DEFAULT_HEIGHT = DEFAULT_WIDTH;

  private:
    size_type m_maxNumber;
    size_type m_gridWidth;
    size_type m_gridHeight;

    static size_type verify(size_type);

  public:
    Geometry(size_type gridWidth=DEFAULT_WIDTH,
             size_type gridHeight=DEFAULT_HEIGHT);

    void setGridWidth(size_type w);
    void setGridHeight(size_type h);

    size_type getMaxNumber() const { return m_maxNumber; }

    size_type getGridWidth() const { return m_gridWidth; }
    size_type getGridHeight() const { return m_gridHeight; }
    size_type getBoardWidth() const { return getMaxNumber(); }
    size_type getGridsPerRow() const { return getGridHeight(); }
    size_type getGridsPerColumn() const { return getGridWidth(); }

    unsigned getRowIndex(unsigned sq) const { return (sq / getMaxNumber()); }
    unsigned getColumnIndex(unsigned sq) const { return (sq % getMaxNumber()); }

    unsigned getGridNumber(unsigned row, unsigned column) const {
      return ( ((row / getGridHeight()) * getGridsPerRow()) +
               (column / getGridWidth()) );
    }

    bitmask_t getNumberMask() const { return getNumberMask( getMaxNumber() ); }
    bitmask_t getSquareMask() const { return getSquareMask( getMaxNumber() ); }

    bool operator==(const Geometry&) const;
    bool operator!=(const Geometry &g) const { return !operator==(g); }

    static size_type getMaxNumber(size_type w, size_type h) { return w * h; }
    static bitmask_t getNumberMask(unsigned n) { return (2 << n) - 2; }
    static bitmask_t getSquareMask(unsigned n) { return (1 << n) - 1; }
  };
}

namespace std
{
  istream& operator>>(istream&, sudoku::Geometry&);
  ostream& operator<<(ostream&, const sudoku::Geometry&);
}

#endif

