#ifndef SEQUENCE_H
#define SEQUENCE_H

#include <iostream>
#include <string>
#include <vector>
#include <list>

#include "bitmask.h"
#include "geometry.h"

namespace sudoku
{
  class Square;
  typedef std::vector<Square*>  SquareSequence;

  class Sequence
  {
  public:
    typedef SquareSequence::size_type  size_type;

  private:
    typedef Bitmask  State;
    typedef std::list<State>  StateStack;
    typedef bool(Square::*SquareFunc)(unsigned);

    std::string m_name;
    SquareSequence m_squares;

    Bitmask m_missing;
    StateStack m_stack;

    const StateStack& getStack() const { return m_stack; }
    StateStack& getStack() { return m_stack; }

    void set(size_type i, Square *sq) { m_squares[i] = sq; }

    void forAllSquares(SquareFunc, unsigned);

  public:
    explicit Sequence(const Geometry&);
    ~Sequence();

    void setName(const std::string &name) { m_name = name; }
    const std::string& getName() const { return m_name; }

    const SquareSequence& getSquares() const { return m_squares; }
    SquareSequence& getSquares() { return m_squares; }

    const Bitmask& getMissing() const { return m_missing; }
    Bitmask& getMissing() { return m_missing; }

    void push();
    bool pop();

    bool add(unsigned number);
    bool add_nothrow(unsigned number);
    bool remove(unsigned number);
    bool isMissing(unsigned number) const { return getMissing().isSet(number); }

    Sequence* findCrossSequence(const Bitmask&) const;

    size_type size() const { return m_squares.size(); }
    bitmask_t getNumberMask() const { return Geometry::getNumberMask(size()); }
    bitmask_t getSquareMask() const { return Geometry::getSquareMask(size()); }

    void dropStack() { m_stack.clear(); }
    void clear();

    Square& operator[](size_type i) { return *getSquare(i, m_squares); }

    const Square& operator[](size_type i) const {
        return *getSquare(i, const_cast<SquareSequence&>(m_squares));
    }

    static Square* getSquare(size_type, SquareSequence&);

    friend class Board;
  };
}

namespace std
{
  ostream& operator<<(ostream&, const sudoku::Sequence&);
  istream& operator>>(istream&, sudoku::Sequence&);
}

#endif
