#include <sstream>
using namespace std;

#include "solution.h"
#include "sequence.h"
#include "square.h"

extern "C"
{
#  include "support.h"
}

namespace std
{

/*********************************************************************
 */
ostream&
operator<<(ostream &ostr, const sudoku::Step &step)
{
  return (ostr << step.message);
}

}


namespace sudoku
{

/*********************************************************************
 */
Step::Step(const Square *s, unsigned n)
  : squareIndex(s->getSquareIndex())
  , number(n)
{}

Step::~Step() {}


/*********************************************************************
 */
Comment::Comment(const string &msg)
  : Step(msg)
{}


/*********************************************************************
 */
ForcedSquare::ForcedSquare(const Square *square, unsigned n)
  : Step(square, n)
{
  ostringstream ostr;
  ostr << _("Forced to set ")
       << square->getName()
       << _(" to ")
       << number
       << ".";

  message = ostr.str();
}


/*********************************************************************
 */
Guesswork::Guesswork(const Square *square, unsigned n)
  : Step(square, n)
{
  ostringstream ostr;
  ostr << _("Proposing setting ")
       << square->getName()
       << _(" to ")
       << number
       << ".";

  message = ostr.str();
}


/*********************************************************************
 */
CompletedSequence::CompletedSequence(const Square *square,
                                     unsigned n,
                                     const Sequence *sequence)
  : Step(square, n)
{
  ostringstream ostr;
  ostr << _("Set ")
       << square->getName()
       << _(" to ")
       << number
       << _(" to complete ")
       << sequence->getName()
       << ".";

  message = ostr.str();
}


/*********************************************************************
 */
NumberSubset::NumberSubset(const Sequence *sequence, const Subset &subset)
{
  unsigned i = subset.getSquares().getFirstBit();
  ostringstream ostr;

  ostr << _("Numbers ") << subset.getNumbers()
       << _(" must go in {")
       << (*sequence)[i].getName();

  while (i != 0)
  {
    if (subset.getSquares().isSet(--i))
      ostr << ", " << (*sequence)[i].getName();
  } /* while */
  
  ostr << _("} in ") << sequence->getName() << ".";

  message = ostr.str();
}


/*********************************************************************
 */
CrossSequenceConstraint::CrossSequenceConstraint(const Square *square,
                                                 unsigned n,
                                                 const Sequence *sequence1,
                                                 const Sequence *sequence2)
  : CompletedSequence(square, 0)
{
  ostringstream ostr;
  ostr << square->getName()
       << _(" CANNOT be ")
       << n
       << _(" in ")
       << sequence2->getName()
       << _(" because that number must appear in ")
       << sequence1->getName()
       << ".";

  message = ostr.str();
}

}

