#ifndef SOLUTION_H
#define SOLUTION_H

#include <iostream>
#include "bitmask.h"

namespace sudoku
{
  class Square;
  class Sequence;

  struct Step
  {
    std::string message;
    unsigned squareIndex;
    unsigned number;

    explicit Step(const std::string &m)
      : message(m)
      , squareIndex(0)
      , number(0) {}
    explicit Step(unsigned i=0, unsigned n=0)
      : squareIndex(i)
      , number(n) {}
    Step(const Square *s, unsigned n);
    ~Step();
  };

  struct Comment : public Step
  {
    explicit Comment(const std::string&);
  };

  struct ForcedSquare : public Step
  {
    ForcedSquare(const Square*, unsigned n);
  };

  struct Guesswork : public Step
  {
    Guesswork(const Square*, unsigned n);
  };

  struct CompletedSequence : public Step
  {
  protected:
    CompletedSequence(const Square *square, unsigned n) : Step(square, n) {}

  public:
    CompletedSequence(const Square*, unsigned n, const Sequence*);
  };

  struct NumberSubset : public Step
  {
    NumberSubset(const Sequence*, const Subset&);
  };

  struct CrossSequenceConstraint : public CompletedSequence
  {
    CrossSequenceConstraint(const Square*,
                            unsigned n,
                            const Sequence*,
                            const Sequence*);
  };
}


namespace std
{
  ostream& operator<<(ostream&, const sudoku::Step&);
}

#endif

