#ifndef STRATEGY_H
#define STRATEGY_H

#include "constants.h"

namespace sudoku
{
  class Puzzle;
  class Freedom;

  class Strategy
  {
  private:
    typedef int(*StrategyFunc)(Puzzle*, const Freedom&);

    StrategyFunc m_strategy;

    explicit Strategy(StrategyFunc func) : m_strategy(func) {}

    static int checkAll(Puzzle*, const Freedom&);
    static int checkCompleteSequences(Puzzle*, const Freedom&);
    static int checkCrossSequenceConstraints(Puzzle*, const Freedom&);
    static int checkNumberSubsets(Puzzle*, const Freedom&);

  public:
    int operator()(Puzzle *puzzle, const Freedom &freedom) const {
      TRACE( if (m_strategy != NULL) ; else abort(); )
      return m_strategy(puzzle, freedom);
    }

    static Strategy ApplyAll;

    static Strategy FindCompleteSequences;
    static Strategy FindCrossSequenceConstraints;
    static Strategy FindNumberSubsets;
  };
}

#endif

